@@ -27,6 +27,7 @@
 #include <QtGui/QStyle>
 #include <QtGui/QStyleOption>
 #include <QtGui/QStyleOptionViewItemV4>
+#include <QtGui/QAbstractItemView>
 #include <QtGui/QTreeView>
 
 #include "entitytreemodel.h"
@@ -47,13 +48,13 @@ enum CountType
 class CollectionStatisticsDelegatePrivate
 {
   public:
-    QTreeView *parent;
+    QAbstractItemView *parent;
     bool drawUnreadAfterFolder;
     DelegateAnimator *animator;
     QColor mSelectedUnreadColor;
     QColor mDeselectedUnreadColor;
 
-    CollectionStatisticsDelegatePrivate( QTreeView *treeView )
+    CollectionStatisticsDelegatePrivate( QAbstractItemView *treeView )
         : parent( treeView ),
           drawUnreadAfterFolder( false ),
           animator( 0 )
@@ -74,7 +75,8 @@ class CollectionStatisticsDelegatePrivate
 
       if ( index.model()->hasChildren( index ) )
       {
-        for ( int row = 0; row < index.model()->rowCount( index ); row++ )
+        const int rowCount = index.model()->rowCount( index );
+        for ( int row = 0; row < rowCount; row++ )
         {
           static const int column = 0;
           count += getCountRecursive<countType>( index.model()->index( row, column, index ) );
@@ -86,6 +88,13 @@ class CollectionStatisticsDelegatePrivate
 
 }
 
+CollectionStatisticsDelegate::CollectionStatisticsDelegate( QAbstractItemView *parent )
+  : QStyledItemDelegate( parent ),
+    d_ptr( new CollectionStatisticsDelegatePrivate( parent ) )
+{
+
+}
+
 CollectionStatisticsDelegate::CollectionStatisticsDelegate( QTreeView *parent )
   : QStyledItemDelegate( parent ),
     d_ptr( new CollectionStatisticsDelegatePrivate( parent ) )
@@ -139,7 +148,9 @@ void CollectionStatisticsDelegate::initStyleOption( QStyleOptionViewItem *option
   QStyleOptionViewItemV4 *noTextOption =
       qstyleoption_cast<QStyleOptionViewItemV4 *>( option );
   QStyledItemDelegate::initStyleOption( noTextOption, index );
-  noTextOption->text.clear();
+  if ( option->decorationPosition != QStyleOptionViewItem::Top ) {
+    noTextOption->text.clear();
+  }
 
   if ( d->animator ) {
 
@@ -203,12 +214,14 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
   // Now calculate the rectangle for the text
   QStyle *s = d->parent->style();
   const QWidget *widget = option4.widget;
-  QRect textRect = s->subElementRect( QStyle::SE_ItemViewItemText, &option4, widget );
+  const QRect textRect = s->subElementRect( QStyle::SE_ItemViewItemText, &option4, widget );
+  const QRect iconRect = s->subElementRect( QStyle::SE_ItemViewItemDecoration, &option4, widget );
 
    // When checking if the item is expanded, we need to check that for the first
   // column, as Qt only recogises the index as expanded for the first column
   QModelIndex firstColumn = index.model()->index( index.row(), 0, index.parent() );
-  bool expanded = d->parent->isExpanded( firstColumn );
+  QTreeView* treeView = qobject_cast<QTreeView*>( d->parent );
+  bool expanded = treeView && treeView->isExpanded( firstColumn );
 
   if ( option.state & QStyle::State_Selected ) {
     painter->setPen( option.palette.highlightedText().color() );
@@ -246,28 +259,36 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
       painter->setFont( font );
     }
 
-    // Squeeze the folder text if it is to big and calculate the rectangles
-    // where the folder text and the unread count will be drawn to
-    QString folderName = text;
-    QFontMetrics fm( painter->fontMetrics() );
-    int unreadWidth = fm.width( unread );
-    if ( fm.width( folderName ) + unreadWidth > textRect.width() ) {
-      folderName = fm.elidedText( folderName, Qt::ElideRight,
-                                  textRect.width() - unreadWidth );
-    }
-    int folderWidth = fm.width( folderName );
-    QRect folderRect = textRect;
-    QRect unreadRect = textRect;
-    folderRect.setRight( textRect.left() + folderWidth );
-    unreadRect.setLeft( folderRect.right() );
-
-    // Draw folder name and unread count
-    painter->drawText( folderRect, Qt::AlignLeft, folderName );
-
     const QColor unreadColor = (option.state & QStyle::State_Selected) ? d->mSelectedUnreadColor : d->mDeselectedUnreadColor;
-    painter->setPen( unreadColor );
-    painter->drawText( unreadRect, Qt::AlignLeft, unread );
 
+    if ( option.decorationPosition == QStyleOptionViewItem::Left
+         || option.decorationPosition == QStyleOptionViewItem::Right ) {
+      // Squeeze the folder text if it is to big and calculate the rectangles
+      // where the folder text and the unread count will be drawn to
+      QString folderName = text;
+      QFontMetrics fm( painter->fontMetrics() );
+      int unreadWidth = fm.width( unread );
+      if ( fm.width( folderName ) + unreadWidth > textRect.width() ) {
+        folderName = fm.elidedText( folderName, Qt::ElideRight,
+                                   textRect.width() - unreadWidth );
+      }
+      int folderWidth = fm.width( folderName );
+      QRect folderRect = textRect;
+      QRect unreadRect = textRect;
+      folderRect.setRight( textRect.left() + folderWidth );
+      unreadRect.setLeft( folderRect.right() );
+
+      // Draw folder name and unread count
+      painter->drawText( folderRect, Qt::AlignLeft, folderName );
+      painter->setPen( unreadColor );
+      painter->drawText( unreadRect, Qt::AlignLeft, unread );
+    } else if ( option.decorationPosition == QStyleOptionViewItem::Top ) {
+      // draw over the icon
+      painter->setPen( unreadColor );
+      if ( unreadCount > 0 ) {
+        painter->drawText( iconRect, Qt::AlignCenter, QString::number( unreadCount ) );
+      }
+    }
     return;
   }
 
@@ -277,8 +298,6 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
 
     PainterStateSaver stateSaver( painter );
 
-    QStyleOptionViewItem opt = option;
-
     QString sumText;
     if ( index.column() == 1 && ( ( !expanded && unreadRecursiveCount > 0 ) || ( expanded && unreadCount > 0 ) ) ) {
       QFont font = painter->font();
@@ -287,7 +306,7 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
       sumText = QString::number( expanded ? unreadCount : unreadRecursiveCount );
     } else {
 
-      qint64 totalCount = statistics.unreadCount();
+      qint64 totalCount = statistics.count();
       qint64 totalRecursiveCount = d->getCountRecursive<TotalCount>( index.sibling( index.row(), 0 ) );
       if (index.column() == 2 && ( ( !expanded && totalRecursiveCount > 0 ) || ( expanded && totalCount > 0 ) ) ) {
         sumText = QString::number( expanded ? totalCount : totalRecursiveCount );
