@@ -22,9 +22,12 @@
 #include "collectionutils_p.h"
 #include "specialcollectionattribute_p.h"
 
+#include "entityhiddenattribute.h"
+
 #include <kdebug.h>
 
 #include <QtCore/QStringList>
+#include <QtGui/QFont>
 
 using namespace Akonadi;
 
@@ -110,8 +113,24 @@ QVariant SubscriptionModel::data(const QModelIndex & index, int role) const
         return true;
       return false;
     }
+    case Qt::FontRole:
+    {
+      const Collection::Id col = index.data( CollectionIdRole ).toLongLong();
+
+      QFont font = CollectionModel::data( index, role ).value<QFont>();
+      font.setBold( d->changes.contains( col ) );
+
+      return font;
+    }
+  }
+
+  if ( role == CollectionIdRole ) {
+    return CollectionModel::data( index, CollectionIdRole );
+  } else {
+    const Collection::Id collectionId = index.data( CollectionIdRole ).toLongLong();
+    const Collection collection = collectionForId( collectionId );
+    return collection.hasAttribute<EntityHiddenAttribute>() ? QVariant() : CollectionModel::data( index, role );
   }
-  return CollectionModel::data( index, role );
 }
 
 Qt::ItemFlags SubscriptionModel::flags(const QModelIndex & index) const
