@@ -18,23 +18,28 @@
     02110-1301, USA.
 */
 
-#include "control.h"
 #include "itemstoretest.h"
+
+#include "control.h"
 #include "testattribute.h"
+#include <akonadi/agentmanager.h>
+#include <akonadi/agentinstance.h>
 #include <akonadi/attributefactory.h>
 #include <akonadi/collectionfetchjob.h>
-#include <akonadi/collectionselectjob.h>
+#include <akonadi/collectionselectjob_p.h>
 #include <akonadi/itemcreatejob.h>
 #include <akonadi/itemdeletejob.h>
 #include <akonadi/itemfetchjob.h>
 #include <akonadi/itemfetchscope.h>
 #include <akonadi/itemmodifyjob.h>
-#include <akonadi/itemmovejob.h>
-#include <qtest_kde.h>
+#include <akonadi/itemmodifyjob_p.h>
+#include <akonadi/resourceselectjob_p.h>
+#include <qtest_akonadi.h>
+#include "test_utils.h"
 
 using namespace Akonadi;
 
-QTEST_KDEMAIN( ItemStoreTest, NoGUI )
+QTEST_AKONADIMAIN( ItemStoreTest, NoGUI )
 
 static Collection res1_foo;
 static Collection res2;
@@ -46,28 +51,22 @@ void ItemStoreTest::initTestCase()
   AttributeFactory::registerAttribute<TestAttribute>();
 
   // get the collections we run the tests on
-  CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
-  QVERIFY( job->exec() );
-  Collection::List list = job->collections();
-  Collection res1;
-  foreach ( const Collection &col, list ) {
-    if ( col.name() == "res1" )
-      res1 = col;
-    if ( col.name() == "res2" )
-      res2 = col;
-    if ( col.name() == "res3" )
-      res3 = col;
-  }
-  foreach ( const Collection &col, list ) {
-    if ( col.name() == "foo" && col.parent() == res1.id() )
-      res1_foo = col;
-  }
+  res1_foo = Collection( collectionIdFromPath( "res1/foo" ) );
+  QVERIFY( res1_foo.isValid() );
+  res2 = Collection( collectionIdFromPath( "res2" ) );
+  QVERIFY( res2.isValid() );
+  res3 = Collection( collectionIdFromPath( "res3" ) );
+  QVERIFY( res3.isValid() );
+
+  // switch all resources offline to reduce interference from them
+  foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) //krazy:exclude=foreach
+    agent.setIsOnline( false );
 }
 
 void ItemStoreTest::testFlagChange()
 {
   ItemFetchJob *fjob = new ItemFetchJob( Item( 1 ) );
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   Item item = fjob->items()[0];
 
@@ -77,10 +76,10 @@ void ItemStoreTest::testFlagChange()
   expectedFlags.insert( "added_test_flag_1" );
   item.setFlag( "added_test_flag_1" );
   ItemModifyJob *sjob = new ItemModifyJob( item, this );
-  QVERIFY( sjob->exec() );
+  AKVERIFYEXEC( sjob );
 
   fjob = new ItemFetchJob( Item( 1 ) );
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
   QCOMPARE( item.flags().count(), expectedFlags.count() );
@@ -91,10 +90,10 @@ void ItemStoreTest::testFlagChange()
   expectedFlags.insert( "added_test_flag_2" );
   item.setFlags( expectedFlags );
   sjob = new ItemModifyJob( item, this );
-  QVERIFY( sjob->exec() );
+  AKVERIFYEXEC( sjob );
 
   fjob = new ItemFetchJob( Item( 1 ) );
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
   QCOMPARE( item.flags().count(), expectedFlags.count() );
@@ -105,10 +104,10 @@ void ItemStoreTest::testFlagChange()
   item.clearFlag( "added_test_flag_1" );
   item.clearFlag( "added_test_flag_2" );
   sjob = new ItemModifyJob( item, this );
-  QVERIFY( sjob->exec() );
+  AKVERIFYEXEC( sjob );
 
   fjob = new ItemFetchJob( Item( 1 ) );
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
   QCOMPARE( item.flags().count(), origFlags.count() );
@@ -120,13 +119,19 @@ void ItemStoreTest::testDataChange_data()
 {
   QTest::addColumn<QByteArray>( "data" );
 
-  QTest::newRow( "empty" ) << QByteArray();
-  QTest::newRow( "nullbyte" ) << QByteArray("\0" );
-  QTest::newRow( "nullbyte2" ) << QByteArray( "\0X" );
+  QTest::newRow( "simple" ) << QByteArray( "testbody" );
+  QTest::newRow( "null" ) << QByteArray();
+  QTest::newRow( "empty" ) << QByteArray( "" );
+  QTest::newRow( "nullbyte" ) << QByteArray( "\0", 1 );
+  QTest::newRow( "nullbyte2" ) << QByteArray( "\0X", 2 );
   QTest::newRow( "linebreaks" ) << QByteArray( "line1\nline2\n\rline3\rline4\r\n" );
   QTest::newRow( "linebreaks2" ) << QByteArray( "line1\r\nline2\r\n\r\n" );
   QTest::newRow( "linebreaks3" ) << QByteArray( "line1\nline2" );
-  QTest::newRow( "simple" ) << QByteArray( "testbody" );
+  QByteArray b;
+  QTest::newRow( "big" ) << b.fill( 'a', 1 << 20 );
+  QTest::newRow( "bignull" ) << b.fill( '\0', 1 << 20 );
+  QTest::newRow( "bigcr" ) << b.fill( '\r', 1 << 20 );
+  QTest::newRow( "biglf" ) << b.fill( '\n', 1 << 20 );
 }
 
 void ItemStoreTest::testDataChange()
@@ -135,55 +140,26 @@ void ItemStoreTest::testDataChange()
 
   Item item;
   ItemFetchJob *prefetchjob = new ItemFetchJob( Item( 1 ) );
-  prefetchjob->exec();
+  AKVERIFYEXEC( prefetchjob );
   item = prefetchjob->items()[0];
   item.setMimeType( "application/octet-stream" );
   item.setPayload( data );
+  QCOMPARE( item.payload<QByteArray>(), data );
 
-  // delete data
+  // modify data
   ItemModifyJob *sjob = new ItemModifyJob( item );
-  QVERIFY( sjob->exec() );
+  AKVERIFYEXEC( sjob );
 
   ItemFetchJob *fjob = new ItemFetchJob( Item( 1 ) );
   fjob->fetchScope().fetchFullPayload();
-  QVERIFY( fjob->exec() );
+  AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
   QVERIFY( item.hasPayload<QByteArray>() );
   QCOMPARE( item.payload<QByteArray>(), data );
-}
-
-void ItemStoreTest::testItemMove()
-{
-  ItemFetchJob *prefetchjob = new ItemFetchJob( Item( 1 ) );
-  prefetchjob->exec();
-  Item item = prefetchjob->items()[0];
-
-  ItemMoveJob *store = new ItemMoveJob( item, res3, this );
-  QVERIFY( store->exec() );
-
-  ItemFetchJob *fetch = new ItemFetchJob( res3, this );
-  QVERIFY( fetch->exec() );
-  QCOMPARE( fetch->items().count(), 1 );
-
-  store = new ItemMoveJob( item, res1_foo, this );
-  QVERIFY( store->exec() );
-}
-
-void ItemStoreTest::testIllegalItemMove()
-{
-  ItemFetchJob *prefetchjob = new ItemFetchJob( Item( 1 ) );
-  prefetchjob->exec();
-  Item item = prefetchjob->items()[0];
-
-  // move into invalid collection
-  ItemMoveJob *store = new ItemMoveJob( item, Collection( INT_MAX ), this );
-  QVERIFY( !store->exec() );
-
-  // move item into folder that doesn't support its mimetype
-  store = new ItemMoveJob( item, res2, this );
-  QEXPECT_FAIL( "", "Check not yet implemented by the server.", Continue );
-  QVERIFY( !store->exec() );
+  QEXPECT_FAIL( "null", "STORE will not update item size on 0 sizes", Continue );
+  QEXPECT_FAIL( "empty", "STORE will not update item size on 0 sizes", Continue );
+  QCOMPARE( item.size(), static_cast<qint64>( data.size() ) );
 }
 
 void ItemStoreTest::testRemoteId_data()
@@ -195,6 +171,7 @@ void ItemStoreTest::testRemoteId_data()
   QTest::newRow( "no-change" ) << QString() << QString( "A" );
   QTest::newRow( "clear" ) << QString( "" ) << QString( "" );
   QTest::newRow( "reset" ) << QString( "A" ) << QString( "A" );
+  QTest::newRow( "utf8" ) << QString( "ä ö ü @" ) << QString( "ä ö ü @" );
 }
 
 void ItemStoreTest::testRemoteId()
@@ -202,26 +179,36 @@ void ItemStoreTest::testRemoteId()
   QFETCH( QString, rid );
   QFETCH( QString, exprid );
 
+  // pretend to be a resource, we cannot change remote identifiers otherwise
+  ResourceSelectJob *rsel = new ResourceSelectJob( "akonadi_knut_resource_0", this );
+  AKVERIFYEXEC( rsel );
+
   ItemFetchJob *prefetchjob = new ItemFetchJob( Item( 1 ) );
-  prefetchjob->exec();
+  AKVERIFYEXEC( prefetchjob );
   Item item = prefetchjob->items()[0];
 
-  item.setId( 1 );
   item.setRemoteId( rid );
   ItemModifyJob *store = new ItemModifyJob( item, this );
-  QVERIFY( store->exec() );
+  store->disableRevisionCheck();
+  store->setIgnorePayload( true ); // we only want to update the remote id
+  AKVERIFYEXEC( store );
 
   ItemFetchJob *fetch = new ItemFetchJob( item, this );
-  QVERIFY( fetch->exec() );
+  AKVERIFYEXEC( fetch );
   QCOMPARE( fetch->items().count(), 1 );
   item = fetch->items().at( 0 );
-  QCOMPARE( item.remoteId(), exprid );
+  QCOMPARE( item.remoteId().toUtf8(), exprid.toUtf8() );
+
+  // no longer pretend to be a resource
+  rsel = new ResourceSelectJob( QString(), this );
+  AKVERIFYEXEC( rsel );
 }
 
 void ItemStoreTest::testMultiPart()
 {
   ItemFetchJob *prefetchjob = new ItemFetchJob( Item( 1 ) );
-  prefetchjob->exec();
+  QVERIFY( prefetchjob->exec() );
+  QCOMPARE( prefetchjob->items().count(), 1 );
   Item item = prefetchjob->items()[0];
   item.setMimeType( "application/octet-stream" );
   item.setPayload<QByteArray>( "testmailbody" );
@@ -237,6 +224,7 @@ void ItemStoreTest::testMultiPart()
   QVERIFY( fjob->exec() );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
+  QVERIFY( item.hasPayload<QByteArray>() );
   QCOMPARE( item.payload<QByteArray>(), QByteArray("testmailbody") );
   QVERIFY( item.hasAttribute<TestAttribute>() );
   QCOMPARE( item.attribute<TestAttribute>()->data, QByteArray("extra") );
@@ -296,27 +284,34 @@ void ItemStoreTest::testRevisionCheck()
   // fetch same item twice
   Item ref( 2 );
   ItemFetchJob *prefetchjob = new ItemFetchJob( ref );
-  prefetchjob->exec();
+  AKVERIFYEXEC( prefetchjob );
+  QCOMPARE( prefetchjob->items().count(), 1 );
   Item item1 = prefetchjob->items()[0];
   Item item2 = prefetchjob->items()[0];
 
   // store first item unmodified
   ItemModifyJob *sjob = new ItemModifyJob( item1 );
-  QVERIFY( sjob->exec() );
+  AKVERIFYEXEC( sjob );
 
-  // try to store second item
+  // store the first item with modifications (should work)
+  item1.attribute<TestAttribute>( Item::AddIfMissing )->data = "random stuff 1";
+  sjob = new ItemModifyJob( item1, this );
+  AKVERIFYEXEC( sjob );
+
+  // try to store second item with modifications (should be detected as a conflict)
+  item2.attribute<TestAttribute>( Item::AddIfMissing )->data = "random stuff 2";
   ItemModifyJob *sjob2 = new ItemModifyJob( item2 );
-  item2.attribute<TestAttribute>( Item::AddIfMissing )->data = "extra";
+  sjob2->disableAutomaticConflictHandling();
   QVERIFY( !sjob2->exec() );
 
   // fetch same again
   prefetchjob = new ItemFetchJob( ref );
-  prefetchjob->exec();
+  AKVERIFYEXEC( prefetchjob );
   item1 = prefetchjob->items()[0];
 
   // delete item
   ItemDeleteJob *djob = new ItemDeleteJob( ref, this );
-  djob->exec();
+  AKVERIFYEXEC( djob );
 
   // try to store it
   sjob = new ItemModifyJob( item1 );
@@ -344,8 +339,8 @@ void ItemStoreTest::testModificationTime()
   item2 = fjob->items().first();
   QCOMPARE( initialDateTime, item2.modificationTime() );
 
-  // Lets wait 5 secs.
-  QTest::qWait( 5000 );
+  // Lets wait at least a second, which is the resolution of mtime
+  QTest::qWait( 1000 );
 
   // Modify the item
   item.attribute<TestAttribute>( Item::AddIfMissing )->data = "extra";
@@ -371,4 +366,25 @@ void ItemStoreTest::testModificationTime()
   ItemDeleteJob *idjob = new ItemDeleteJob( item, this );
   QVERIFY( idjob->exec() );
 }
+
+void ItemStoreTest::testRemoteIdRace()
+{
+  // Create an item and store it
+  Item item;
+  item.setMimeType( "text/directory" );
+  ItemCreateJob *job = new ItemCreateJob( item, res1_foo );
+  QVERIFY( job->exec() );
+
+  // Fetch the same item again. It should not have a remote Id yet, as the resource
+  // is offline.
+  // The remote id should be null, not only empty, so that item modify jobs with this
+  // item don't overwrite the remote id.
+  Item item2( job->item().id() );
+  ItemFetchJob *fetchJob = new ItemFetchJob( item2 );
+  QVERIFY( fetchJob->exec() );
+  QCOMPARE( fetchJob->items().size(), 1 );
+  QVERIFY( fetchJob->items().first().remoteId().isNull() );
+}
+
+
 #include "itemstoretest.moc"
